# This file was auto-generated by Fern from our API Definition.

from ....core.client_wrapper import SyncClientWrapper
from .raw_client import RawApplicationsClient
import typing
from ....core.request_options import RequestOptions
from ....core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawApplicationsClient


class ApplicationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawApplicationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawApplicationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawApplicationsClient
        """
        return self._raw_client

    def get_pod_template_hash_to_deployment_version(
        self, id: str, *, pod_template_hashes: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, float]:
        """
        This endpoint fetches the pod template hash to deployment version map for a specific application.

        Parameters
        ----------
        id : str
            Id of the application

        pod_template_hashes : str
            Pod Template Hashes (comma separated for multiple)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, float]
            Successfully retrieved the pod template hash to deployment version map.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.v1.internal.applications.get_pod_template_hash_to_deployment_version(
            id="id",
            pod_template_hashes="podTemplateHashes",
        )
        """
        response = self._raw_client.get_pod_template_hash_to_deployment_version(
            id, pod_template_hashes=pod_template_hashes, request_options=request_options
        )
        return response.data


class AsyncApplicationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawApplicationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawApplicationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawApplicationsClient
        """
        return self._raw_client

    async def get_pod_template_hash_to_deployment_version(
        self, id: str, *, pod_template_hashes: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, float]:
        """
        This endpoint fetches the pod template hash to deployment version map for a specific application.

        Parameters
        ----------
        id : str
            Id of the application

        pod_template_hashes : str
            Pod Template Hashes (comma separated for multiple)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, float]
            Successfully retrieved the pod template hash to deployment version map.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.v1.internal.applications.get_pod_template_hash_to_deployment_version(
                id="id",
                pod_template_hashes="podTemplateHashes",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get_pod_template_hash_to_deployment_version(
            id, pod_template_hashes=pod_template_hashes, request_options=request_options
        )
        return response.data
